/**
 * ExplorerModel.java
 * @author Takeshi Yamada
 * @version
 */

import java.util.Vector;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeModel;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.table.TableModel;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.event.EventListenerList;

import javax.swing.table.DefaultTableModel;

public class ExplorerModel extends DefaultTreeSelectionModel implements TreeModel, TableModel {
    
    private EventListenerList listeners = new EventListenerList();
    private FileSystemModel model;
    private Class[] columnClasses = { String.class, Long.class };

    public ExplorerModel(FileNode root) {
        model = FileSystemModel.getInstance();
        model.setRootNode(root);
    }

    public void setCurrentNode(FileNode current) {
        model.setCurrentNode(current);
        fireTableChanged(new TableModelEvent(this));
    }

    // TreeModel
    public void addTreeModelListener(TreeModelListener listener) {
        listeners.add(TreeModelListener.class, listener);
    }

    public int getIndexOfChild(Object parent, Object child) {
        return model.getIndexOfChildNode((FileNode)parent, (FileNode)child);
    }

    public Object getChild(Object parent, int index) {
        return model.getChildNode((FileNode)parent, index);
    }

    public int getChildCount(Object parent) {
        return ((FileNode)parent).getChildCount();
    }

    public Object getRoot() {
        return model.getRootNode();
    }

    public void addFileNode(FileNode parent, FileNode child) {
        int index = parent.getChildCount();
        model.addFileNode(parent, child, index);
        Object[] path = getPathToRoot(child);
        int[] indexes = new int[]{index};
        int newIndex = index + 1;
        Object[] newChildren = new Object[newIndex];
        for (int i = 0; i < newIndex - 1; i++) {
            newChildren[i] = parent.getChild(i);
        }
        fireTreeNodesInserted(parent, path, indexes, newChildren);
    }

    public boolean isLeaf(Object o) {
        return ((FileNode)o).isLeaf();
    }

    public void removeTreeModelListener(TreeModelListener l) {
        listeners.remove(TreeModelListener.class, l);
    }


    public void valueForPathChanged(TreePath path, Object newValue) {
    }

    public void addTableModelListener(TableModelListener l) {
        listeners.add(TableModelListener.class, l);
    }

    public Class getColumnClass(int columnIndex) {
        return columnClasses[columnIndex];
    }

    public int getColumnCount() {
        return columnClasses.length;
    }

    public String getColumnName(int columnIndex) {
        return model.getColumnNames()[columnIndex];
    }

    public int getRowCount() {
        FileNode current = model.getCurrentNode();
        if (current == null) return 0;
        return model.getChildNodeCount(current);
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        Object[][] infos = model.getCurrentNodeChildrenInfos();
        if (infos != null) {
            return infos[rowIndex][columnIndex];
        }
        return null;
    }

    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return false;
    }

    public void removeTableModelListener(TableModelListener l) {
        listeners.remove(TableModelListener.class, l);
    }


    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {}


    // DefaultTreeSelectionModel
    public void setSelectionPath(TreePath path) {
        super.setSelectionPath(path);
        setCurrentNode((FileNode)path.getLastPathComponent());
    }

    protected void fireTreeNodesChanged(Object source, 
                                        Object[] path, 
                                        int[] childIndices, 
                                        Object[] children) {

        Object[] ls = listeners.getListenerList();
        TreeModelEvent e = null;
        for (int i = 0; i < ls.length; i++) {
            if (ls[i] == TreeModelListener.class) {
            }
        }
    }
                      
    protected void fireTreeNodesInserted(Object source, 
                                         Object[] path, 
                                         int[] childIndices, 
                                         Object[] children) {

        Object[] ls = listeners.getListenerList();
        TreeModelEvent e = null;
        for (int i = ls.length - 2 ; i>=0; i-=2) {
            if (ls[i] == TreeModelListener.class) {
                e = new TreeModelEvent(source, path, childIndices, children);
                ((TreeModelListener)ls[i+1]).treeNodesInserted(e);
            }
        }
    }
                      
    protected void fireTreeNodesRemoved(Object source, 
                                        Object[] path, 
                                        int[] childIndices, 
                                        Object[] children) {
    }

    protected void fireTreeStructureChanged(Object source, 
                                            Object[] path, 
                                            int[] childIndices, 
                                            Object[] children) {

    }

    private Object[] getPathToRoot(FileNode node) {
        Vector v = new Vector();
        FileNode root = model.getRootNode();
        for  (FileNode fNode = node.getParent(); fNode != root; fNode = fNode.getParent()) {
            v.add(0, fNode);
        }
        v.add(0, root);
        return v.toArray();
    }

    public void fireTableChanged(TableModelEvent e) {
        if (listeners == null) return;
        Object[] ls = listeners.getListenerList();
        for (int i = ls.length-2; i>=0; i-=2) {
            if (ls[i]==TableModelListener.class) {
                ((TableModelListener)ls[i+1]).tableChanged(e);
            }
        }
    }

} // ExplorerModel.java





